# -*- coding: cp1252 -*-
"""
MacDING

this is the main file which is called when the application
is executed

TODO:

NOTE:
	the variable names 'german' and 'english' are due to historical reasons.
	lang1/lang2 would be more appropriate.
"""
#import pdb  #debug

import objc
from Foundation import *
from AppKit import *
from PyObjCTools import NibClassBuilder, AppHelper
import time

ENCODING = 'utf8'

# get the classes defined in the nib-file 
NibClassBuilder.extractClasses("MainMenu")

# import the modules and make their classes available
import dingPrefs
import dingSearch


# class defined in Main.nib
class SearchController(NibClassBuilder.AutoBaseClass):
    # the actual base class is NSObject
    # The following outlets are added to the class:
    # queryField	- user supplied search term
    # resultField	- NSTableView object for displaying results
    # search		- button
	# spinningWheel	- the status wheel
	# statusText	- small next next to the status wheel
	# AeButton		- -Button
	# aeButton		- -Button
	# OeButton		- -Button
	# oeButton		- -Button
	# UeButton		- -Button
	# ueButton		- -Button
	# szButton		- -Button
	# customGrepField - user supplied grep command
	# searchModeMatrix - which searchmode? (matrix of radiobuttons)
	# grepCmdText	- gives the user information about the applied command
	data = {'german':[],
			'english':[]}
	
	def awakeFromNib(self):
		# set the data source for the table (the object itself)
		self.resultField.setDataSource_(self)
		cols = self.resultField.tableColumns()
		for col in cols:
			cell = col.dataCell()
			cell.setWraps_(True)
		self.queryField.window().makeKeyAndOrderFront_(self)
		self.queryField.selectText_(self)

	def resetData(self):
		self.data={'german':[], 'english':[]}
	
	def search_(self, sender):
		"""Method is connected to the Search Button"""
		# get the string out of the search field
		term = self.queryField.stringValue()
		if not term: return	# don't query if string is empty
		term = term.encode(ENCODING)

		# set another search command
		if self.prefs.SEARCHMODE==0:
			self.prefs.saveDefaultSearch(self.searchModeMatrix.selectedCell().title(), self.customGrepField)
			self.prefs.setGrepCmd(self.searchModeMatrix.selectedCell().title(), self.customGrepField)
			self.grepCmdText.setStringValue_("Cmd: %s"%self.prefs.GREPCMD%term)
		else:
			self.grepCmdText.setStringValue_("Warning: grep not found! Using internal search")

		# perform the search in an extra class
		# timing and animation wheel
		self.spinningWheel.startAnimation_(sender)
		bt = time.time()
		# search!
		found = self.search.performSearch(term)
		et = time.time()
		self.statusText.setStringValue_("%.2f sec/%i items"%(float(et-bt), len(found)))
		self.spinningWheel.stopAnimation_(sender)

		# Align text in resultField according to RTL/LTR stuff in file
		if self.prefs.LANGUAGES[0]['textflow']=="RTL":
			self.resultField.tableColumnWithIdentifier_("first").dataCell().setAlignment_(1)
		else:
			self.resultField.tableColumnWithIdentifier_("first").dataCell().setAlignment_(0)
		if self.prefs.LANGUAGES[1]['textflow']=="RTL":
			self.resultField.tableColumnWithIdentifier_("second").dataCell().setAlignment_(1)
		else:
			self.resultField.tableColumnWithIdentifier_("second").dataCell().setAlignment_(0)
		

		# insert the found items in the table to be
		# displayed (here the dictionary data!)
		self.resetData()
		for f in found:
			# skipt commentaries
			if f.startswith('#'): continue
			try: german, english = f.split('::')
			except: print "Warning: wrong line format"; continue
			wl = german.split('|'); tl = english.split('|')
			if len(wl) != len(tl):
				ger = unicode(german, ENCODING, 'replace')
				eng = unicode(english, ENCODING, 'replace')
				if self.prefs.LANGUAGES[0]['textflow']=='RTL':
					ger = ger.rjust(len(ger))
				if self.prefs.LANGUAGES[1]['textflow']=='RTL':
					eng = eng.rjust(len(eng))
				self.data['german'].append(ger)
				self.data['english'].append(eng)
			else:
				ger = unicode(wl.pop(), ENCODING, 'replace')
				eng = unicode(tl.pop(), ENCODING, 'replace')
				if self.prefs.LANGUAGES[0]['textflow']=='RTL':
					ger = ger.rjust(len(ger))
				if self.prefs.LANGUAGES[1]['textflow']=='RTL':
					eng = eng.rjust(len(eng))
				self.data['german'].append(ger)
				self.data['english'].append(eng)
				while len(wl) > 0:
					ger = unicode(wl.pop(), ENCODING, 'replace')
					eng = unicode(tl.pop(), ENCODING, 'replace')
					if self.prefs.LANGUAGES[0]['textflow']=='RTL':
						ger = ger.rjust(len(ger))
					if self.prefs.LANGUAGES[1]['textflow']=='RTL':
						eng = eng.rjust(len(eng))
					self.data['german'].append("\t"+ger)
					self.data['english'].append("\t"+eng)
		# mark the text after finishing search
		self.queryField.selectText_(sender)
		self.resultField.reloadData()


	# umlaut function (connected to the umlaut buttons in the drawer
	def insert_(self, sender):
		"""an Umlaut-Button has been pressed"""
		c = ''
		# get the state of the buttons
		if self.aeButton.state():
			self.aeButton.setState_(0)
			c = u''
		elif self.AeButton.state():
			self.AeButton.setState_(0)
			c = u''		
		elif self.oeButton.state():
			self.oeButton.setState_(0)
			c = u''
		elif self.OeButton.state():
			self.OeButton.setState_(0)
			c = u''
		elif self.ueButton.state():
			self.ueButton.setState_(0)
			c = u''
		elif self.UeButton.state():
			self.UeButton.setState_(0)
			c = u''
		elif self.szButton.state():
			self.szButton.setState_(0)
			c = u''
		# append the pressed button-value to the query-string
		if c:
			self.queryField.setStringValue_(self.queryField.stringValue()+c)
	

	# table view delegate methods 
	def numberOfRowsInTableView_(self, tableView):
		if self.data is None:
			return 0
		return max(len(self.data['german']), len(self.data['english']))
	def tableView_objectValueForTableColumn_row_(self, tableView, col, row):
		#col = col.headerCell().stringValue()
		if col.identifier()=="first": col="german"
		else: col="english"
		return str(self.data[col][row])
	def tableView_shouldEditTableColumn_row_(self, tableView, col, row):
		return 0

if __name__ == "__main__":
    AppHelper.runEventLoop()
