"""
dingPref.py

File controls the preferences window
"""
import objc
from Foundation import *
from AppKit import *
from PyObjCTools import NibClassBuilder, AppHelper
import os

DEBUG=1
ENCODING = 'utf8'

class PreferencesController(NibClassBuilder.AutoBaseClass):
	# outlets:
	#	main		- main window search controller instance
	# 	window 		- preference window
	# 	dictPopUp	- popUp for dictionaries in drawer!
	#	dictTable	- NSTable object containing dicts
		
	# configuration
	ACTIVE_DICT= ''
	DICTS = []
	GREPCMD='grep -h "%s" '+ACTIVE_DICT
	SEARCHMODE = 0	# 0 - external grep
					# 1 - internal re-search
	CONFIGDIR = os.path.expanduser('./config')
	CONFIGFILE = os.path.join(CONFIGDIR, 'config.txt')
	LANGUAGES =[{'name':'', 'textflow':''}, 
				{'name':'', 'textflow':''}]

	
	# -------------- PRIVATE HELPERS ------------------ #

	# change some configurations when using an arabian language-dictionary
	#	- LTR ouput
	#	- right-alignment
	#	- double spaced table rows, increased font size
	def __is_arabian(self, dictfile):
		if self.LANGUAGES[0]['name']=="persian" or self.LANGUAGES[1]['name']=="arabian" or\
			self.LANGUAGES[1]['name']=="persian" or self.LANGUAGES[0]['name']=="arabian":
			if DEBUG: print "Is Arabian"
			return 1
		else:
			return 0

	def __set_arab_config(self):
		self.main.resultField.setRowHeight_(32)

	def __set_default_config(self):
		self.main.resultField.setRowHeight_(16)
	
	def set_languages(self):
		import os
		grep = os.popen("grep -h \#\ Languages %s"%self.ACTIVE_DICT)
		lines = grep.readlines(); grep.close()
		# no languages-tag in dictionary -> using default
		if len(lines)==0: 
			self.LANGUAGES[0]['name']="Language 1"
			self.LANGUAGES[1]['name']="Language 2"
		else:
			p1, p2 = lines[0].split("<->")
			l1 = p1.strip().split(" ").pop().strip()
			l2 = p2.strip().split(" ")[0].strip()
			self.LANGUAGES[0]['name']=l1
			self.LANGUAGES[1]['name']=l2
		# set the header string to the appropriate value
		self.main.resultField.tableColumnWithIdentifier_("first").headerCell().setStringValue_(self.LANGUAGES[0]['name'])
		self.main.resultField.tableColumnWithIdentifier_("second").headerCell().setStringValue_(self.LANGUAGES[1]['name'])

		# deal with the textflow thingy
		grep = os.popen("grep -h \#\ Text\-Alignment %s"%self.ACTIVE_DICT)
		lines = grep.readlines(); grep.close()
		# default is LTR for both languages
		if len(lines)==0:
			self.LANGUAGES[0]['textflow']="LTR"
			self.LANGUAGES[1]['textflow']="LTR"
		else:
			try:
				p1, p2 = lines[0].split("/")
			except:
				print "Warning, dictionary header corrupted"
			else:
				l1 = p1.strip().split(" ").pop().strip()
				l2 = p2.strip().split(" ")[0].strip()
				self.LANGUAGES[0]['textflow']=l1
				self.LANGUAGES[1]['textflow']=l2
		if DEBUG: print self.LANGUAGES
		return

	def saveDefaultSearch(self, button, customcmd):
		dswritten=0
		cgwritten=0
		if os.path.isfile(self.CONFIGFILE):
			if DEBUG: print "Saving: ", button, customcmd.stringValue()
			f = open(self.CONFIGFILE, "r")
			lines = f.readlines(); f.close()
			f = open(self.CONFIGFILE, "w")
			for line in lines:
				if line.startswith("default_search"):
					f.write("default_search=%s\n"%button)
					dswritten=1
				elif line.startswith("custom_grep"):
					f.write("custom_grep=%s\n"%customcmd.stringValue())
					cgwritten=1
				else:
					f.write(line)
			if not dswritten: 
				f.write("default_search=%s\n"%button)
			if not cgwritten:
				f.write("custom_grep=%s\n"%customcmd.stringValue())
			f.close()
		else:
			f.open(self.CONFIGFILE, "w")
			f.write("default_search=%s\n"%button)
			f.write("custom_grep=%s\n"%customcmd.stringValue())
			f.close()
			
	def restoreDefaultSearch(self):
		def title2tag(title):
			if title=='normal': return 0
			elif title=='ignore case': return 1
			elif title=='word in text': return 2
			elif title=='word at beginning': return 3
			elif title=='regular expression': return 4
			elif title=='custom': return 5
			else: return -1
		default_search=''
		custom_grep=''
		if os.path.isfile(self.CONFIGFILE):
			f = open(self.CONFIGFILE, "r")
			lines = f.readlines(); f.close()
			for line in lines:
				if line.startswith("default_search"):
					default_search=line.split("=")[1].strip()
				elif line.startswith("custom_grep"):
					custom_grep=line.split("=")[1].strip()
		if default_search:
			self.main.searchModeMatrix.selectCellWithTag_(title2tag(default_search))
		if custom_grep:
			self.main.customGrepField.setStringValue_(custom_grep)
			

	# -------------- ACTIONS -------------------------- #	
	def awakeFromNib(self):
		"""executed when extracted from NIB"""
		if not os.path.isdir(self.CONFIGDIR):
			os.mkdir(self.CONFIGDIR)
		self.findDictionaries()
		self.testForGrep()
		self.initializeDictPopUp()
		self.dictTable.setDataSource_(self)
		self.set_languages()
		if self.__is_arabian(self.ACTIVE_DICT): 
			self.__set_arab_config()
		else:
			self.__set_default_config()
		self.restoreDefaultSearch()

	def switchDict_(self, sender):
		"""called in the drawer"""
		selected_index = self.dictPopUp.indexOfSelectedItem()
		self.ACTIVE_DICT=self.DICTS[selected_index]

		self.set_languages()
		if self.__is_arabian(self.ACTIVE_DICT): 
			self.__set_arab_config()
		else:
			self.__set_default_config()
		# save the default dictionary in configfile
		ddwritten=0
		if os.path.isfile(self.CONFIGFILE):
			f = open(self.CONFIGFILE, "r")
			lines = f.readlines(); f.close()
			f = open(self.CONFIGFILE, 'w')
			for line in lines:
				if line.startswith("default_dict="):
					f.write("default_dict=%s\n"%self.ACTIVE_DICT)
					ddwritten=1
				else:
					f.write(line)
			if not ddwritten:
				f.write("default_dict=%s\n"%self.ACTIVE_DICT)
			f.close()
		else:
			f = open(self.CONFIGFILE, 'w')
			f.write("default_dict=%s\n"%self.ACTIVE_DICT)
			f.close()





	def addDict_(self, sender):
		"""add a dictionary to the system"""
		# open the file selection objec
		od = NSOpenPanel.openPanel()
		# display the dialog
		result = od.runModalForDirectory_file_types_(None, None, ['ding'])
		
		# else the cancel Button was pressed
		if not result: return
		file = od.filenames()[0]
		self.writeToConfig('dictionary', file)
		self.findDictionaries()
		self.initializeDictPopUp()
		self.dictTable.reloadData()
	
	def delDict_(self, sender):
		"""delete a dictionary from the list"""
		deldict = self.dictTable.selectedRow()
		dictstring = "dictionary="+self.DICTS[deldict]
		self.delFromConfig(dictstring)
		self.findDictionaries()
		self.initializeDictPopUp()
		self.dictTable.reloadData()

	def reveal_(self):
		"""show pref window"""
		self.dictTable.reloadData()
		self.window.makeKeyAndOrderFront_(self)
	# --------------------------------------- #		

	def delFromConfig(self, string):
		"""deletes string from configfile if found"""
		if not os.path.isfile(self.CONFIGFILE): return
		f = open(self.CONFIGFILE, 'r')
		lines = f.readlines()
		f.close()
		f = open(self.CONFIGFILE, 'w')
		for line in lines:
			if not line.startswith(string):
				f.write(line)
		return

	def writeToConfig(self,id, content, mode='several'):
		"""write id and content to file: id=content"""
		newline = "%s=%s\n"%(id, content)
		if not os.path.isfile(self.CONFIGFILE):
			f = open(self.CONFIGFILE, 'w')
			f.write(newline)
			f.close()
		else:
			f = open(self.CONFIGFILE, 'r')
			lines = f.readlines()
			f.close()
			for line in lines:
				if line == newline:
					return
			if mode =='unique':
				f = open(self.CONFIGFILE, 'w')
				for line in lines:
					if not line.startswith(id):
						f.write(line)
				f.write(newline)	
				f.close()
			else:
				f = open(self.CONFIGFILE, 'a')
				f.write(newline)
				f.close()
			

	def initializeDictPopUp(self):
		"""fills the dictPopup with the appropriate values"""
		self.dictPopUp.removeAllItems()
		self.dictPopUp.addItemWithTitle_(os.path.splitext(os.path.basename(self.ACTIVE_DICT))[0])
		for dict in self.DICTS:
			if dict == self.ACTIVE_DICT: continue
			title = os.path.splitext(os.path.basename(dict))[0]
			if self.dictPopUp.itemWithTitle_(title):
				title+=" (2)"
			self.dictPopUp.addItemWithTitle_(title)
		# if no config file preference, use first dict in list
		default_dict=os.path.splitext(os.path.basename(self.DICTS[0]))[0]
		if os.path.isfile(self.CONFIGFILE):
			f = open(self.CONFIGFILE, "r")
			lines = f.readlines(); f.close()
			for line in lines:
				if line.startswith("default_dict"):
					default_dict=line.split("=")[1].strip()
					default_dict=os.path.splitext(os.path.basename(default_dict))[0]
					if DEBUG: print default_dict
		# set the popup list to the users default item from config file
		self.dictPopUp.selectItem_(self.dictPopUp.itemWithTitle_(default_dict))
		self.switchDict_(None)
		if self.__is_arabian(self.ACTIVE_DICT): 
			self.__set_arab_config()
		else:
			self.__set_default_config()
	
	def testForGrep(self):
		"""tests if grep is available and sets searchmode accordingly"""
		path = os.environ['PATH']
		paths = path.split(':')
		found = 0
		for p in paths:
			if os.path.isfile(os.path.join(p, 'grep')):
				found = 1
				break
		if found:
			self.SEARCHMODE=0
		else:
			self.SEARCHMODE=1

	def findDictionaries(self):
		"""build self.DICTS list"""
		import glob
		# directories to look for dictionaries
		search_dirs = ['.',\
						os.path.join(os.path.expanduser('~'), '.dictionaries'),\
						os.path.join('/Libraries', 'Dictionaries'),
						os.path.join(os.path.expanduser('~'), 'Library', 'Dictionaries')]
		
		l = []
		for dir in search_dirs:
			g = glob.glob(os.path.join(dir, '*.ding'))
			if g: l += g
		self.DICTS = l

		if os.path.isfile(self.CONFIGFILE):
			f = open(self.CONFIGFILE, 'r')
			lines = f.readlines()
			f.close()
			for line in lines:
				if line.startswith('dictionary='):
					path = line.split('=')[1][:-1]
					if path not in self.DICTS:
						if os.path.isfile(path): 
							self.DICTS.append(path)
		if self.DICTS: self.ACTIVE_DICT=l[0]
	
	def setGrepCmd(self, keyword, customGrepField):
		""" set the Grep command for searching"""
		if keyword=='normal': 
			self.GREPCMD='grep -h "%s" '+self.ACTIVE_DICT
		elif keyword=='ignore case':
			self.GREPCMD='grep -h -i "%s" '+self.ACTIVE_DICT
		elif keyword=='word in text':
			self.GREPCMD='grep -h -i " %s " '+self.ACTIVE_DICT
		elif keyword=='word at beginning':
			self.GREPCMD='grep -h -E "(^|:: )%s" '+self.ACTIVE_DICT
		elif keyword=='regular expression':
			self.GREPCMD='grep -h -E "%s" '+self.ACTIVE_DICT
		elif keyword=='custom':
			cmd = customGrepField.stringValue()
			self.GREPCMD=cmd+self.ACTIVE_DICT
			
	# table view delegate methods 
	def numberOfRowsInTableView_(self, tableView):
		if self.DICTS is None:
			return 0
		return len(self.DICTS)
	def tableView_objectValueForTableColumn_row_(self, tableView, col, row):
		colname = col.headerCell().stringValue()
		d = self.DICTS[row]
		path = os.path.dirname(d)
		dict = os.path.splitext(os.path.basename(d))[0]
		if colname == 'path':
			return path
		else:
			return dict
	def tableView_shouldEditTableColumn_row_(self, tableView, col, row):
		return 0

